#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include "windows.h"
#endif

#include "math.h"

#include "BatAnimation.h"


BatEffectA::BatEffectA(int n,float r,float g,float b,float min_speed,float max_speed,int length,GLTManager *GLTM)
{
	int i;

	m_ba1 = m_ba2 = m_ba3 = 0;

	m_cycle = 0;
	m_length = length;
	m_r = r;
	m_b = b;
	m_g = g;
	m_GLTM = GLTM;

	m_bat_animations.Delete();

	for(i=0;i<n;i++) {
		BatAnimation *bat = new BatAnimation();
		bat->x = float(rand()%640);
		bat->y = float(rand()%480);

		{
			float vx = bat->x-320;
			float vy = bat->y-240;
			float n = float(sqrt(vx*vx+vy*vy));
//			float n = 320;
			if (n>0) {
				vx /= n;
				vy /= n;
			} else {
				vx = 0;
				vy = 1;
			} // if

			bat->speed_x = (min_speed+((rand()%1000)/1000.0f)*(max_speed-min_speed))*vx;
			bat->speed_y = (min_speed+((rand()%1000)/1000.0f)*(max_speed-min_speed))*vy;

			bat->speed_t = 0.8f+0.4f*((rand()%1000)/1000.0f);
			bat->scale = 1.5f+1.0f*((rand()%1000)/1000.0f);
		}

		bat->t = float(rand()%100);
		
		m_bat_animations.Add(bat);
	} // for
}


bool BatEffectA::cycle()
{
	BatAnimation *bat;
	List<BatAnimation> to_delete;
//	float f = (m_cycle)/25.0f;
//	if (f>1.0f) f = 1.0f;
	float f = 1.0f;

	m_bat_animations.Rewind();
	while(m_bat_animations.Iterate(bat)) {
		bat->x+=bat->speed_x*f;
		bat->y+=bat->speed_y*f;
		bat->t+=bat->speed_t;

		if (bat->x<-16 || bat->x>=656 || bat->y<=-16 || bat->y>=496) to_delete.Add(bat);
	} // while 

	while(!to_delete.EmptyP()) {
		bat = to_delete.ExtractIni();
		m_bat_animations.DeleteElement(bat);
		delete bat;
	} // while 

	m_cycle++;
	if (m_cycle>=m_length) return false;

	return !(m_bat_animations.EmptyP());
}


void BatEffectA::draw()
{
	BatAnimation *bat;
	int t;
	float f = 0.0;

	if (m_ba1==0) {
		m_ba1 = m_GLTM->get("bat-animation1");
		m_ba2 = m_GLTM->get("bat-animation2");
		m_ba3 = m_GLTM->get("bat-animation3");
	} // if

	if (m_cycle<25) {
		float f;

        glEnable(GL_COLOR_MATERIAL);

		f = (25-m_cycle)/25.0f;

        glColor4f(m_r, m_g, m_b, f);
        glNormal3f(0.0, 0.0, 1.0);

        glBegin(GL_QUADS);
        glVertex3f(0, 0, 0);
        glVertex3f(0, 480, 0);
        glVertex3f(640, 480, 0);
        glVertex3f(640, 0, 0);
        glEnd();
	}

	f = 1.0f - (m_cycle/float(m_length));

	m_bat_animations.Rewind();
	while(m_bat_animations.Iterate(bat)) {
		t = int(bat->t)%20;
		if (t<5) m_ba1->draw(m_r, m_g, m_b,f, bat->x,bat->y, 0,0,bat->scale);
		if (t>=5 && t<10) m_ba2->draw(m_r, m_g, m_b,f, bat->x,bat->y, 0,0,bat->scale);
		if (t>=10 && t<15) m_ba1->draw(m_r, m_g, m_b,f, bat->x,bat->y, 0,0,bat->scale);
		if (t>=15) m_ba3->draw(m_r, m_g, m_b,f, bat->x,bat->y, 0,0,bat->scale);
	} // while 
}



BatEffectB::BatEffectB(int n,float r,float g,float b,float min_speed,float max_speed,int length,GLTManager *GLTM)
{
	int i;

	m_ba1 = m_ba2 = m_ba3 = 0;

	m_cycle = 0;
	m_length = length;
	m_r = r;
	m_b = b;
	m_g = g;
	m_GLTM = GLTM;

	for(i=0;i<n;i++) {
		BatAnimation *bat = new BatAnimation();
		switch(rand()%4) {
		case 0:
				bat->x = float(rand()%640);
				bat->y = -16;
				break;
		case 1:
				bat->x = float(rand()%640);
				bat->y = 496;
				break;
		case 2:
				bat->x = -16;
				bat->y = float(rand()%480);
				break;
		case 3:
				bat->x = 656;
				bat->y = float(rand()%480);
				break;
		}

		{
			float vx = 320-bat->x;
			float vy = 240-bat->y;
//			float n = float(sqrt(vx*vx+vy*vy));
			float n = 320;
			if (n>0) {
				vx /= n;
				vy /= n;
			} else {
				vx = 0;
				vy = 1;
			} // if

			bat->speed_x = (min_speed+((rand()%1000)/1000.0f)*(max_speed-min_speed))*vx;
			bat->speed_y = (min_speed+((rand()%1000)/1000.0f)*(max_speed-min_speed))*vy;

			bat->speed_t = 0.8f+0.4f*((rand()%1000)/1000.0f);
			bat->scale = 1.5f+1.0f*((rand()%1000)/1000.0f);
		}

		bat->t = float(rand()%100);
		
		m_bat_animations.Add(bat);
	} // for
}


bool BatEffectB::cycle()
{
	BatAnimation *bat;
	List<BatAnimation> to_delete;
//	float f = (m_cycle)/25.0f;
//	if (f>1.0f) f = 1.0f;
	float f = 1.0f;
	float d;

	m_bat_animations.Rewind();
	while(m_bat_animations.Iterate(bat)) {
		bat->x+=bat->speed_x*f;
		bat->y+=bat->speed_y*f;
		bat->t+=bat->speed_t;

		d = (bat->x-320)*(bat->x-320) + (bat->y-240)*(bat->y-240);

		if (d<256) to_delete.Add(bat);
	} // while 

	while(!to_delete.EmptyP()) {
		bat = to_delete.ExtractIni();
		m_bat_animations.DeleteElement(bat);
		delete bat;
	} // while 

	m_cycle++;
	if (m_cycle>=m_length) return false;

	return !(m_bat_animations.EmptyP());
}


void BatEffectB::draw()
{
	BatAnimation *bat;
	int t;
	float f = 0.0;

	if (m_ba1==0) {
		m_ba1 = m_GLTM->get("bat-animation1");
		m_ba2 = m_GLTM->get("bat-animation2");
		m_ba3 = m_GLTM->get("bat-animation3");
	} // if

	{
		float bx = 320*(m_cycle/float(m_length));
		float by = 240*(m_cycle/float(m_length));

        glEnable(GL_COLOR_MATERIAL);

        glColor4f(m_r, m_g, m_b, 1);
        glNormal3f(0.0, 0.0, 1.0);

        glBegin(GL_QUADS);
        glVertex3f(0, 0, 0);
        glVertex3f(0, by, 0);
        glVertex3f(640, by, 0);
        glVertex3f(640, 0, 0);

        glVertex3f(0, 0, 0);
        glVertex3f(0, 480, 0);
        glVertex3f(bx, 480, 0);
        glVertex3f(bx, 0, 0);

        glVertex3f(640-bx, 0, 0);
        glVertex3f(640-bx, 480, 0);
        glVertex3f(640, 480, 0);
        glVertex3f(640, 0, 0);

        glVertex3f(0, 480-by, 0);
        glVertex3f(0, 480, 0);
        glVertex3f(640, 480, 0);
        glVertex3f(640, 480-by, 0);
        glEnd();
	}

	f = 1.0f - (m_cycle/float(m_length));

	m_bat_animations.Rewind();
	while(m_bat_animations.Iterate(bat)) {
//		d = (bat->x-320)*(bat->x-320) + (bat->y-240)*(bat->y-240);
//		if (d<4096) {
//			f = d/4096.0f;
//		} else {
			f = 1.0f;
//		} 
		t = int(bat->t)%20;
		if (t<5) m_ba1->draw(m_r, m_g, m_b,f, bat->x,bat->y, 0,0,bat->scale);
		if (t>=5 && t<10) m_ba2->draw(m_r, m_g, m_b,f, bat->x,bat->y, 0,0,bat->scale);
		if (t>=10 && t<15) m_ba1->draw(m_r, m_g, m_b,f, bat->x,bat->y, 0,0,bat->scale);
		if (t>=15) m_ba3->draw(m_r, m_g, m_b,f, bat->x,bat->y, 0,0,bat->scale);
	} // while 
}
